@@ -42,7 +42,6 @@ void update_betas (field<vec> &betas, mat &res_betas, field<vec> &acceptance_bet
                    const mat &Wlong_bar, const mat &Wlong_sds,
                    const uvec &id_H_, const uvec &id_h,
                    const field<uvec> &FunForms,
-                   const field<uvec> &FunForms_ind,
                    const List Funs_FunForms,
                    const vec &alphas,
                    const bool &any_event, const bool &any_interval,
@@ -52,7 +51,9 @@ void update_betas (field<vec> &betas, mat &res_betas, field<vec> &acceptance_bet
                    const uvec &id_H_fast, const uvec &id_h_fast,
                    const uvec &which_event, const uvec &which_right_event, const uvec &which_left,
                    const uvec &which_interval, const field<uvec> &unq_idL,
-                   const uword &n_burnin) {
+                   const uword &n_burnin,
+                   const bool &recurrent,
+                   const vec &frailtyH_sigmaF_alphaF, const vec &frailtyh_sigmaF_alphaF) {
   uword n_b = b_mat.n_rows;
   // FE in HC - Gibbs sampling
   vec betas_vec = docall_rbindF(betas);
@@ -99,18 +100,18 @@ void update_betas (field<vec> &betas, mat &res_betas, field<vec> &acceptance_bet
   // update logLik_surv
   Wlong_H =
     calculate_Wlong(X_H, Z_H, U_H, Wlong_bar, Wlong_sds, betas, b, id_H_,
-                    FunForms, FunForms_ind, Funs_FunForms);
+                    FunForms, Funs_FunForms);
   WlongH_alphas = Wlong_H * alphas;
   if (any_event) {
     Wlong_h =
       calculate_Wlong(X_h, Z_h, U_h, Wlong_bar, Wlong_sds, betas, b, id_h,
-                      FunForms, FunForms_ind, Funs_FunForms);
+                      FunForms, Funs_FunForms);
     Wlongh_alphas = Wlong_h * alphas;
   }
   if (any_interval) {
     Wlong_H2 =
       calculate_Wlong(X_H2, Z_H2, U_H2, Wlong_bar, Wlong_sds, betas, b,
-                      id_H_, FunForms, FunForms_ind, Funs_FunForms);
+                      id_H_, FunForms, Funs_FunForms);
     WlongH2_alphas = Wlong_H2 * alphas;
   }
   logLik_surv = log_surv(W0H_bs_gammas, W0h_bs_gammas, W0H2_bs_gammas,
@@ -118,7 +119,8 @@ void update_betas (field<vec> &betas, mat &res_betas, field<vec> &acceptance_bet
                          WlongH_alphas, Wlongh_alphas, WlongH2_alphas,
                          log_Pwk, log_Pwk2, id_H_fast, id_h_fast,
                          which_event, which_right_event, which_left,
-                         any_interval, which_interval);
+                         any_interval, which_interval,
+                         recurrent, frailtyH_sigmaF_alphaF, frailtyh_sigmaF_alphaF);
 
   ///////////////////////////////////////////////////////////////////////////////
   // FE outside HC - Metropolis-Hastings sampling
@@ -159,14 +161,14 @@ void update_betas (field<vec> &betas, mat &res_betas, field<vec> &acceptance_bet
         // logLik_surv proposal
         mat Wlong_H_prop =
           calculate_Wlong(X_H, Z_H, U_H, Wlong_bar, Wlong_sds, betas_prop, b,
-                          id_H_, FunForms, FunForms_ind, Funs_FunForms);
+                          id_H_, FunForms, Funs_FunForms);
         vec WlongH_alphas_prop = Wlong_H_prop * alphas;
         mat Wlong_h_prop(Wlong_h.n_rows, Wlong_h.n_cols);
         vec Wlongh_alphas_prop(Wlongh_alphas.n_rows);
         if (any_event) {
           Wlong_h_prop =
             calculate_Wlong(X_h, Z_h, U_h, Wlong_bar, Wlong_sds, betas_prop,
-                            b, id_h, FunForms, FunForms_ind, Funs_FunForms);
+                            b, id_h, FunForms, Funs_FunForms);
           Wlongh_alphas_prop = Wlong_h_prop * alphas;
         }
         mat Wlong_H2_prop(Wlong_H2.n_rows, Wlong_H2.n_cols);
@@ -174,7 +176,7 @@ void update_betas (field<vec> &betas, mat &res_betas, field<vec> &acceptance_bet
         if (any_interval) {
           Wlong_H2_prop =
             calculate_Wlong(X_H2, Z_H2, U_H2, Wlong_bar, Wlong_sds, betas_prop,
-                            b, id_H_, FunForms, FunForms_ind, Funs_FunForms);
+                            b, id_H_, FunForms, Funs_FunForms);
           WlongH2_alphas_prop = Wlong_H2_prop * alphas;
         }
         vec logLik_surv_prop =
@@ -183,7 +185,8 @@ void update_betas (field<vec> &betas, mat &res_betas, field<vec> &acceptance_bet
                    WlongH_alphas_prop, Wlongh_alphas_prop, WlongH2_alphas_prop,
                    log_Pwk, log_Pwk2, id_H_fast, id_h_fast,
                    which_event, which_right_event, which_left,
-                   any_interval, which_interval);
+                   any_interval, which_interval,
+                   recurrent, frailtyH_sigmaF_alphaF, frailtyh_sigmaF_alphaF);
         // numerator
         double numerator_j =
           sum_logLik_long_j_prop + sum(logLik_surv_prop) + logPrior_j_prop;
